          SUBROUTINE SECOWA(PTIME,PTARR)
C         ------------------------------
C
C         PTIME = Total time, which equals
C           PTARR(1) user time plus
C           PTARR(2) system time.
C
          DIMENSION PTARR(2)
          DOUBLE PRECISION TDBL
C
          CALL TIMER(TDBL)
          PTARR(1)=TDBL
          PTARR(2)=0.0
          PTIME=PTARR(1)
C
          END
