C     ***************
      SUBROUTINE HEAD(
C     ***************
     >                NS,NT,NPX,NPY,NPZ,NPT,NP,NW1,
     >                START,IAVG,IAVGDD,SEED,ITEMAX,
     >                METHIT,ITBLUP,ORDEXP,BETA,SIGMA )
C
C
C     SUBROUTINE HEAD PRINTS THE HEADING OF THE OUTPUT FILE
C
C
C     LATEST REVISION - NOV. 1989; E.KEHL
C
C
      DOUBLE PRECISION  BETA,SIGMA
C
C
      INTEGER   ORDEXP,SEED,START
C
C
      CHARACTER*10 PST(0:1)
C
C
      DATA PST /'DISORDERED',
     >          '   ORDERED'/
C
C
      CALL HEADER(NW1)
C
      WRITE( NW1, 1010)
      WRITE( NW1, 1020) NS,NT
      WRITE( NW1, 1030) NPX,NPY,NPZ,NPT,NP
      WRITE( NW1, 1040) PST(START)
      WRITE( NW1, 1050) ITEMAX
      WRITE( NW1, 1055) METHIT
      WRITE( NW1, 1060) IAVGDD
      WRITE( NW1, 1070) IAVG
      WRITE( NW1, 1080) SEED
      WRITE( NW1, 1090) ITBLUP
      WRITE( NW1, 1100) SIGMA
      WRITE( NW1, 1110) BETA
      WRITE( NW1, 1120)
C
C
 1010 FORMAT(2(/),15X,
     >'INPUT PARAMETER:'/15X,
     >'--------------- '     )
C
 1020 FORMAT(//5X,
     >'LATTICE SIZE                            :'
     >,I3,' CUBED TIMES ',I3)
C
 1030 FORMAT(5X,
     >'NUMBER OF PROCESSES IN X-DIMENSION      :',9X,I10/5X,
     >'NUMBER OF PROCESSES IN Y-DIMENSION      :',9X,I10/5X,
     >'NUMBER OF PROCESSES IN Z-DIMENSION      :',9X,I10/5X,
     >'NUMBER OF PROCESSES IN T-DIMENSION      :',9X,I10/5X,
     >'TOTAL NUMBER OF PROCESSES               :',9X,I10)
C
 1040 FORMAT(5X,
     >'START CONFIGURATION                     :',9X,A10)
C
 1050 FORMAT(5X,
     >'TOTAL NUMBER OF ITERATION               :',9X,I10)
C
 1055 FORMAT(5X,
     >'NUMBER OF METROPOLIS-HITS               :',9X,I10)
 1060 FORMAT(5X,
     >'DISCARDED ITERATIONS FOR THERMALIZATION :',9X,I10)
C
 1070 FORMAT(5X,
     >'MODUL FOR TAKING AVERAGES               :',9X,I10)
C
 1080 FORMAT(5X,
     >'START VALUE OF RANDOM NUMBER GENERATOR  :',9X,I10)
C
 1090 FORMAT(5X,
     >'SU3 MATRIX TABLE UPDATE MODUL           :',9X,I10)
C
 1100 FORMAT(5X,
     >'STANDARD DEVIATION OF GAUSSIAN RAND. NO.:',4X,E15.6)
C
 1110 FORMAT(5X,
     >'BARE COUPLING CONSTANT                  :',4X,E15.6)
C
 1120 FORMAT(2(/),15X,
     >'RESULTS:'/15X,
     >'------- '     )
C
C
      END
