
      SUBROUTINE INIT( PSI, PSIDOT, RANDOM, NCOLOR)  
C     ---------------------------------------------

      INTEGER VSIZE, NCOLOR
      COMMON /VECLEN/ VSIZE

      DOUBLE PRECISION  PSI(VSIZE), PSIDOT(VSIZE), RANDOM(VSIZE)

      CALL INITSH

      IF (NCOLOR .EQ. 3) THEN
        CALL INISU3 
      ELSE
        PRINT *, ' ERROR ! invalid value of ncolor -', NCOLOR,
     *           ' - in subroutine init'
        STOP
      ENDIF

      PRINT *, 'initsu finished'

      CALL INITPD( PSI, PSIDOT, RANDOM)
      CALL INIPSI( PSI, PSIDOT)

      END
