
      SUBROUTINE COPVEC (V, U)  
C     -------------------------
      INTEGER VSIZE
      COMMON /VECLEN/ VSIZE
      DOUBLE PRECISION   U(VSIZE), V(VSIZE)

C     ------------------------------- 
C     |        Set {v} = {u}        | 
C     ------------------------------- 
      INTEGER I

      DO 8  I = 1, VSIZE
        V (I) = U (I) 
   8  CONTINUE
      END
