      INTEGER     PROCX, PROCY, PROCZ, NT, NX, SEGT, SEGX, SEGY, SEGZ,
     *            NSITES, NCMPLX, NCOLOR, VSIZE
      PARAMETER ( NT = 8,  NX = 8, PROCX = 1,  PROCY = 2,  PROCZ = 2,
     *            SEGX = NX/PROCX,  SEGY = NX/PROCY,  SEGZ = NX/PROCZ,
     *            SEGT = NT,        NSITES = SEGT * SEGX * SEGY * SEGZ,
     *            NCMPLX = 2,  NCOLOR = 3,  VSIZE = NSITES * NCOLOR )

C   nt      :-  number of lattice sites in the t -direction (must be even)
C   nx      :-  number of lattice sites in the x, y & z -directions (even)
C   procx   :-  number of processsors in the x -direction
C   procy   :-  number of processsors in the y -direction
C   procz   :-  number of processsors in the z -direction
C   ncolor  :-  should always be 3 for SU(3) simulation

