
      SUBROUTINE SUBVEC (DIFF, U, V)  
C     ------------------------------
      INTEGER VSIZE
      COMMON /VECLEN/ VSIZE
      DOUBLE PRECISION  DIFF(VSIZE), U(VSIZE), V(VSIZE)

C     -------------------------------- 
C     |      {diff} = {u} - {v}      | 
C     --------------------------------
      INTEGER I

      DO 8  I = 1, VSIZE
        DIFF (I) = U (I) - V (I)
   8  CONTINUE
      END
