C     *******************
      REAL FUNCTION GETCL()
C     *******************

      INTEGER FIRST
      DOUBLE PRECISION TIMEI, TIME
      SAVE TIMEI, FIRST
      DATA FIRST/0/

      IF (FIRST .EQ. 0) THEN

         CALL TIMER (TIMEI)
         GETCL = 0.0
         FIRST = 1

      ELSE

	 CALL timer(TIME)
         GETCL = (TIME - TIMEI) 

      ENDIF

      END
