      INTEGER     NT, NX, SEGT, SEGX, SEGY, SEGZ, NSITES, NCMPLX,
     *            NCOLOR, VSIZE
      PARAMETER ( NT = 8,  NX = 4, SEGX = NX,  SEGY = NX,  SEGZ = NX,
     *            SEGT = NT,        NSITES = SEGT * SEGX * SEGY * SEGZ,
     *            NCMPLX = 2,  NCOLOR = 3,  VSIZE = NSITES * NCOLOR )

C   nt      :-  number of lattice sites in the t -direction
C   nx      :-  number of lattice sites in the x, y & z -directions
C   ncolor  :-  should always be 3 for SU(3) simulation
