      INTEGER  SEEDMS, RESMSG,  LOCMSG,  XMSG,    XYMSG,   TOTMSG,
     *         POXMSG, POYMSG, POZMSG, NEXMSG, NEYMSG, NEZMSG,
     *         PARMSG, TSKMSG, TIMEMS

      PARAMETER ( SEEDMS = 1, RESMSG = 2,  LOCMSG = 3, XMSG = 4,
     * XYMSG = 5, TOTMSG = 6,  POXMSG = 7, POYMSG = 8, POZMSG = 9,
     * NEXMSG = 10, NEYMSG = 11, NEZMSG =12, PARMSG = 13, 
     * TSKMSG = 14 , TIMEMS = 15)


